/*
 * Copyright 2012-2016 Canonical Ltd.
 *
 * This file is part of messaging-app.
 *
 * messaging-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * messaging-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.2
import Ubuntu.Components 1.3
import Ubuntu.Contacts 0.1
import QtContacts 5.0

Page {
    id: newRecipientPage
    objectName: "newRecipientPage"

    property Item multiRecipient: null
    property string phoneToAdd: ""
    property QtObject contactIndex: null

    function moveListToContact(contact)
    {
        if (active) {
            newRecipientPage.contactIndex = null
            contactList.positionViewAtContact(contact)
        } else {
            newRecipientPage.contactIndex = contact
        }
    }

    function addRecipient(phoneNumber)
    {
        multiRecipient.addRecipient(phoneNumber)
        multiRecipient.forceActiveFocus()
        mainStack.removePage(newRecipientPage)
    }

    header: PageHeader {
        id: pageHeader

        property alias leadingActions: leadingBar.actions
        property alias trailingActions: trailingBar.actions

        title: i18n.tr("Add recipient")
        leadingActionBar {
            id: leadingBar
        }

        trailingActionBar {
            id: trailingBar
            actions: [
                Action {
                    text: i18n.tr("Back")
                    iconName: "back"
                    onTriggered: {
                        mainStack.removePage(newRecipientPage)
                        newRecipientPage.destroy()
                    }
                }
            ]
        }
    }

    Sections {
        id: headerSections
        model: contactMap.hasFavorites || (contactList.mostCalledCount > 0) ?
                                    [ i18n.tr("All"), i18n.tr("Favorites") ] : []
        onModelChanged: selectedIndex = 0
        onSelectedIndexChanged: {
            switch (selectedIndex) {
            case 0:
                contactList.showAllContacts()
                break;
            case 1:
                contactList.showFavoritesContacts()
                break;
            default:
                break;
            }
        }
    }

    TextField {
        id: searchField

        anchors {
            left: parent.left
            leftMargin: units.gu(2)
            right: parent.right
            rightMargin: units.gu(2)
            topMargin: units.gu(1.5)
            bottomMargin: units.gu(1.5)
            verticalCenter: parent.verticalCenter
        }
        onTextChanged: newRecipientPage.currentIndex = -1
        inputMethodHints: Qt.ImhNoPredictiveText
        placeholderText: i18n.tr("Search...")
        visible: false
    }

    state: "default"
    states: [
        State {
            id: defaultState
            name: "default"
            property list<QtObject> trailingActions: [
                Action {
                    text: i18n.tr("Search")
                    iconName: "search"
                    onTriggered: {
                        newRecipientPage.state = "searching"
                        contactList.showAllContacts()
                        searchField.forceActiveFocus()
                    }
                }
            ]

            PropertyChanges {
                target: pageHeader
                trailingActions: defaultState.trailingActions
                extension: headerSections
            }
            PropertyChanges {
                target: searchField
                text: ""
                visible: false
            }
        },
        State {
            id: searchingState
            name: "searching"
            property list<QtObject> leadingActions: [
                Action {
                    iconName: "back"
                    text: i18n.tr("Cancel")
                    onTriggered: {
                        newRecipientPage.forceActiveFocus()
                        newRecipientPage.state = "default"
                        headerSections.selectedIndex = 0
                    }
                }
            ]

            PropertyChanges {
                target: pageHeader
                leadingActions: searchingState.leadingActions
                trailingActions: []
                contents: searchField
            }

            PropertyChanges {
                target: headerSections
                visible: false
            }

            PropertyChanges {
                target: searchField
                text: ""
                visible: true
            }
        }
    ]

    ContactMap {
        id: contactMap
    }

    ContactListView {
        id: contactList
        objectName: "newRecipientList"
        anchors {
            top: pageHeader.bottom
            left: parent.left
            right: parent.right
            bottom: keyboard.top
        }

        showAddNewButton: true
        showImportOptions: (contactList.count === 0) && (filterTerm == "")

        filterTerm: searchField.text
        onContactClicked: {
            if (newRecipientPage.phoneToAdd != "") {
                mainView.addPhoneToContact(newRecipientPage,
                                           contact,
                                           newRecipientPage.phoneToAdd,
                                           newRecipientPage,
                                           contactList.listModel)
            } else {
                mainView.showContactDetails(newRecipientPage,
                                            contact,
                                            newRecipientPage,
                                            contactList.listModel)
            }
        }

        onAddNewContactClicked: {
            var newContact = ContactsJS.createEmptyContact(newRecipientPage.phoneToAdd, newRecipientPage)
            mainStack.addFileToCurrentColumnSync(newRecipientPage,
                                             Qt.resolvedUrl("MessagingContactEditorPage.qml"),
                                             { model: contactList.listModel,
                                               contact: newContact,
                                               initialFocusSection: (newRecipientPage.phoneToAdd != "" ? "phones" : "name"),
                                               contactListPage: newRecipientPage })
        }
    }

    Component.onCompleted: {
        if (QTCONTACTS_PRELOAD_VCARD !== "") {
            contactList.listModel.importContacts("file://" + QTCONTACTS_PRELOAD_VCARD)
        }
    }
    onActiveChanged: {
        if (active && (state === "searching")) {
            searchField.forceActiveFocus()
        }
    }

    KeyboardRectangle {
        id: keyboard
    }

    Connections {
        target: contactList.listModel
        onContactsChanged: {
            if (newRecipientPage.contactIndex) {
                contactList.positionViewAtContact(newRecipientPage.contactIndex)
                newRecipientPage.contactIndex = null
            }
        }
    }
}
